use super::*;

pub trait FunctorContext<'a>: 'a {
    type T: WeakFunctor<'a>;
}

pub trait FunctorContextExt<'a>: FunctorContext<'a> {
    fn fmap<A: 'a, B: 'a>(
        fa: Wrap<'a, A, Self::T>,
        f: impl 'a + FnOnce(A) -> B,
    ) -> Wrap<'a, B, Self::T>
    where
        Self::T: Functor<'a>,
    {
        <Self::T as Functor>::fmap(fa, f)
    }

    fn pure<A: 'a>(a: A) -> Wrap<'a, A, Self::T>
    where
        Self::T: Pure<'a>,
    {
        <Self::T as Pure>::pure(a)
    }

    fn bind<A: 'a, B: 'a>(
        fa: Wrap<'a, A, Self::T>,
        f: impl 'a + FnOnce(A) -> Wrap<'a, B, Self::T>,
    ) -> Wrap<'a, B, Self::T>
    where
        Self::T: Monad<'a>,
    {
        <Self::T as Monad>::bind(fa, f)
    }

    fn fail<A: 'a, E: 'a>(e: E) -> Wrap<'a, A, Self::T>
    where
        Self::T: Fail<'a, E>,
    {
        <Self::T as Fail<_>>::fail(e)
    }
}

impl<'a, Ctx: FunctorContext<'a>> FunctorContextExt<'a> for Ctx {}
